

Application create: #AspectBrowser with:
        (#( EtTools MaterialCluster)
            collect: [:each | Smalltalk at: each ifAbsent: [
                self error: 'Not all of the prerequisites are loaded']])!

AspectBrowser becomeDefault!
CwExtendedPrompter subclass: #AboutAspectBrowserPrompter
    instanceVariableNames: 'buttonOK icon '
    classVariableNames: ''
    poolDictionaries: ''!

AspectBrowser becomeDefault!
CwExtendedPrompter subclass: #CwStandardPrompter
    instanceVariableNames: 'buttonOK buttonCancel '
    classVariableNames: ''
    poolDictionaries: ''!

AspectBrowser becomeDefault!
CwStandardPrompter subclass: #AspectPrompter
    instanceVariableNames: 'nameText '
    classVariableNames: ''
    poolDictionaries: ''!

AspectBrowser becomeDefault!
CwStandardPrompter subclass: #CompositeAspectPrompter
    instanceVariableNames: 'aspectsList newAspectList nameText '
    classVariableNames: ''
    poolDictionaries: ''!

AspectBrowser becomeDefault!
CwStandardPrompter subclass: #LibraryUpdatePrompter
    instanceVariableNames: 'materialList aspectList materialClass aspectClass differences '
    classVariableNames: ''
    poolDictionaries: ''!

AspectBrowser becomeDefault!
CwStandardPrompter subclass: #MaterialUpdatePrompter
    instanceVariableNames: 'materialsList changedAspect '
    classVariableNames: ''
    poolDictionaries: ''!

AspectBrowser becomeDefault!
Application subclass: #AspectBrowser
    instanceVariableNames: 'shell materialList matAspectList methodsList aspectsList sourceText materialMethodsButton aspectMethodsButton fileMenu textMenu aspectMenu methodMenu methodsSupplier confirmClose addButtonPixmap removeButtonPixmap appIcon '
    classVariableNames: ''
    poolDictionaries: 'CwConstants CgConstants '!

AspectBrowser becomeDefault!

!AboutAspectBrowserPrompter privateMethods !

bulletinBoardArgs
	"Answer an argBlock to use when creating the bulletinBoardWidget."

	^[:w |
		w
			marginWidth: 10;
			marginHeight: 10;
			verticalSpacing: 4;
			horizontalSpacing: 4;
			mappedWhenManaged: true ]
!

buttonOk: w clientData: clientData callData: callData
	"Perform the callback processing for buttonOk activate."

	icon freeIcon.
	self ok: w clientData: clientData callData: callData
!

createBulletinBoard
	"Create the bulletinBoardWidget which is the immediate child of the shell.
	 This is a Form for instances of the receiver and subclasses."

	self bulletinBoardWidget: (self shell createForm: 'form' argBlock: self bulletinBoardArgs).!

createChildren
	"Create, initialize and manage the children of the bulletinBoardWidget."

	|  bb iconLabel info1Label info2Label info3Label |

	bb := self bulletinBoardWidget.

	icon := self createIcon.
	iconLabel := bb
		createLabel: 'iconLabel'
		argBlock: [ :w |
			w
				labelType: XmICON;
				labelIcon: icon ].
	iconLabel manageChild.

	info1Label := bb
		createLabel: 'info1Label'
		argBlock: nil.
	info1Label labelString: 'Aspect Browser for the Tools and Materials Framework'.
	info1Label manageChild.

	info2Label := bb
		createLabel: 'info2Label'
		argBlock: nil.
	info2Label labelString: 'Version 0.2'.
	info2Label manageChild.

	info3Label := bb
		createLabel: 'info3Label'
		argBlock: nil.
	info3Label labelString: 'Copyright  1995  UBILAB / UBS'.
	info3Label manageChild.

	buttonOK := bb
		createPushButton: 'OK'
		argBlock: [ :w |
			w width: 65 ].
	buttonOK
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonOk:clientData:callData:
		clientData: nil.
	buttonOK manageChild.

	bb defaultButton: buttonOK.

	iconLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHFORM ].

	info1Label setValuesBlock: [ :w |
		w
			leftAttachment: XmATTACHWIDGET;
			leftWidget: iconLabel;
			rightAttachment: XmATTACHFORM ].

	info2Label setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: info1Label;
			topOffset: 0;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: iconLabel ].
	
	info3Label setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: info2Label;
			topOffset: 0;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: iconLabel ].

	buttonOK setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: info3Label;
			topOffset: 15;
			bottomAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHPOSITION;
			leftPosition: 40 ].
!

createIcon
	"Creates the icon from a .ICO file"

	| format icons fileName |

	format := CgWinICOFileFormat new.
	fileName := self iconPath, 'aspect.ico'.
	icons := format loadFromFile: fileName.
	format hasErrorOccurred
		ifTrue: [ 
			Transcript cr; show: 'Could not load icon ', fileName.
			^CgIcon fromResource: 'oti'. ].

	^icons first.
!

iconPath

	^'C:\IBMST\bitmaps\'
!

shellArgs
	"Answer an argBlock to use as the create args for the receiver's shell."

	^[:w | w
		title: self title;
		mwmDecorations: MWMDECORMENU | MWMDECORBORDER | MWMDECORTITLE ]!

title
	"Answer the window title to use for the receiver."

	^'About Aspect Browser'
! !


!AspectBrowser class publicMethods !

new
	
	^super new initialize.
! !


!AspectBrowser publicMethods !

open
	"Opens the application window and initializes the menu
	and the widget tree"

	| main menuBar |

	appIcon := self createAppIcon.
	shell := CwTopLevelShell 
		createApplicationShell: 'shell'
		argBlock: [ :w | 
			w 
				title: 'Aspect Browser';
				icon: appIcon. ].

	main := shell 
		createMainWindow: 'main' 
		argBlock: nil.
	main manageChild.

	self confirmClose: false. 
	menuBar := self createMenuBar: main.
	main 
		setAreas: (menuBar createMenuBarFor: main owner: self) manageChild
		horizontalScrollbar: nil
		verticalScrollbar: nil
		workRegion: nil.

	self createWindow: main.

	shell realizeWidget.
! !

!AspectBrowser privateMethods !

acceptText: aString from: requestor
	"Accept aString if possible. The action performed depends
	 on the current value of the textUpdateSelector. If a compile
	 error occurs, notify the requestor."

	| supplierClass newMethod category |

	(methodsSupplier = #Aspect) 
		ifTrue: [ 
			supplierClass := self selectedAspect.
			category := supplierClass name asString. ]
		ifFalse: [ 
			| aspect |
			supplierClass := self selectedMaterial. 
			aspect := self selectedMaterialAspect.
			aspect notNil 
				ifTrue: [ category := aspect name asString. ]
				ifFalse: [ category := supplierClass name asString. ]].

	supplierClass notNil 
		ifTrue: [
			self execShortOperation: [
				newMethod := self 
					compile: aString 
					inClass: supplierClass 
					inCategory: category
					notifying: requestor.
				newMethod isNil ifTrue: [ ^false ].
				self selectedMethod isNil
					ifTrue: [ self refreshMethods. ]
					ifFalse: [ 
						self selectedMethod selector ~= newMethod selector 
							ifTrue: [ self refreshMethods. ]].
				^true. ]]
		ifFalse: [ 
			System message: 'No aspect selected. Text not saved.'.
			^false ].
!

addAspectButtonPressed: widget clientData: clientData callData: callData
	"The Add Aspect button has been pressed."

	self addAspectToMaterial.
!

addAspectToMaterial
	"Copy the selected aspect in the aspect
	list box to the selected material class"

	| materialClass aspectClass |

	materialClass := self selectedMaterial.
	materialClass notNil 
		ifTrue: [
			aspectClass := self selectedAspect.
			aspectClass notNil 
				ifTrue: [
					(materialClass hasAspect: aspectClass)
						ifFalse: [
							(materialClass hasAspectInSuperclass: aspectClass)
								ifTrue: [
									(System confirm: 'The aspect ', aspectClass printString, ' is contained in a superclass of ',
										materialClass printString, '. Do you realy want to overwrite the aspect?')
										ifFalse: [ ^self. ]].
							self execShortOperation: [
								aspectClass addToMaterial: materialClass.
								self refreshMaterialAspects ]]
						ifTrue: [ 
							System message: 'The material ', materialClass name asString, ' allready has the aspect ', 
							aspectClass name asString, ', perhaps as a part of a composite aspect.' ]]
				ifFalse: [ System message: 'No aspect selected.' ].
		]
		ifFalse: [ System message: 'No material class selected.' ].
!

addValueChangedCallback: widget 
	"Add the valueChangedCallback and clear the modified flag."

	widget 
		addCallback: XmNvalueChangedCallback
		receiver: self
		selector: #valueChanged:clientData:callData:
		clientData: nil.
	self confirmClose: false.!

aspectClasses
	"Returns all aspect classes directly derived from class Aspect.
	Composit aspects are denoted with '...' " 

	| aspects |

	aspects := (SimpleAspect subclasses collect: [ :cl | cl name asString ]) asSortedCollection.
	aspects addAll: (CompositeAspect subclasses collect: [ :cl | cl name asString, '...' ]).
	^aspects!

aspectDoubleSelect: widget clientData: clientData callData: callData
	"An entry in the Aspects list box was double clicked."

	self 
		hierarchyListDoubleSelect: widget
		selectedItem: self selectedAspect
		childrenSelector: #baseAspectClasses
		asStringSelector: #printString
		callData: callData
!

aspectMenu
	"Answer a menu of operations for the Aspects widget."

	^CwMenu new
		title: '~Aspect';
		subMenuSelector: #menuAspect;
		add: #menuAspectNew  label: '~New...' enable: true for: nil;
		add: #menuAspectNewComposite      label: 'New ~Composite...' enable: true for: nil;
		add: #menuAspectDelete          label: '~Delete...'  enable: [self enableAspectDelete] for: nil;
		add: #menuAspectRename    label: '~Rename...'  enable: [self enableAspectRename] for: nil;
		addLine;
		add: #menuAspectAddToMaterial     	label: '~Add to Material' enable: [self enableAspectAddToMaterial] for: nil;
		add: #menuAspectUpdateMaterials 	label: '~Update Materials...' enable: [self enableAspectUpdate] for: nil;
		yourself
!

aspectMethods: aspectClass

	| methods |

	methods := aspectClass aspectMethods.
	methods notNil 
		ifTrue: [
			methods := methods collect: [ :m | m selector asString ].
			^methods asSortedCollection]
		ifFalse: [ ^nil ].
!

aspectMethodsPressed: widget clientData: clientData callData: callData
	"The Library aspect methods button has been pressed."

	callData set
		ifTrue: [
			self changeRequest ifFalse: [
				widget set: false.
				materialMethodsButton set: true.
				^self].
			methodsSupplier := #Aspect.
			self refreshMethods.
			methodsList deselectAllItems.
			self changeContentsOf: sourceText to: ''. ].
!

aspectSelected: widget clientData: clientData callData: callData
	"An entry in the Aspects list box was selected.
	Put the method selectors of the selected aspect in the Methods list box."

	(methodsSupplier = #Aspect) 
		ifTrue: [
			self refreshMethods.
			self changeContentsOf: sourceText to: ''. ].
!

bitmapPath

	^'bitmaps\'
!

changeContentsOf: textWidget to: aString
	"Private - Change the text in the textWidget to be aString.
	 If aString is nil the text will not be set and the modified
	 state will not be cleared."

	aString == nil ifTrue: [^self].
	textWidget
		deferRedraw: [
			textWidget
				setInsertionPosition: 0;
				showPosition: 0;
				setString: aString] ;
		updateWidget.
	self confirmClose = true ifTrue: [ self addValueChangedCallback: textWidget] !

changeRequest
	"Answer true if it is OK to change the window contents." 

	| result |

	self confirmClose ~~ true ifTrue: [ ^ true]. 
	result := (CwMessagePrompter for: shell)
		iconType: XmICONQUESTION ;
		title: 'Unsaved Changes' ;
		buttonType: XmYESNOCANCEL ;
		messageString: 'Text has been modified - save changes?' ;
		prompt.

	result = false
		ifTrue:[
			"Don't save changes - set text change flag to 'unchanged' and proceed"
			self resetTextChangeFlag: sourceText.
			^true
		].

	result == nil
		ifTrue:[
			"Cancel - don't proceed"
			^false
		].

	"Otherwise save the changes and proceed.
	 NOTE: self save answers false if the save was unsuccessful."
	^self saveChanges!

compile: source inClass: aClass inCategory: categoryString notifying: requestor
	"Private - Answer the method which results from compiling the source in
	aClass.  Notify requestor if the compiler detects an error. Fail if aClass is a
	shadow class.  Answer nil if an error occurred.
	The method is categorized in a category with the name categoryString."

	| resident result newCategories selectedMethod private application |

	((resident := Smalltalk classAt: aClass symbol) isNil or: [
		aClass ~~ ((aClass isKindOf: Class) ifTrue: [resident] ifFalse: [resident class])]) ifTrue: [
			^System message: 'The method cannot be saved because its class (', aClass name,
				') is a shadow class.\\Only loaded class editions may be modified.' addLineDelimiters].

	(result := aClass basicCompile: source notifying: requestor ifFail: [nil]) isNil ifTrue: [^nil].

	selectedMethod := self selectedMethod.
	selectedMethod isNil 
		ifTrue: [ 
			private := false.
			application := self selectedAspect applications first ]
		ifFalse: [ 
			private := selectedMethod isPrivate.
			application := selectedMethod application ].
	newCategories := Array with: categoryString.
	result := result methodClass
		install: result
		asPrivate: private
		withSource: source
		ifNewAddTo:  application
		categorizeIn: newCategories.

	result isNil ifTrue: [^nil].

	EtTools recordMethodCreation ifTrue: [
		EtTools managerInterface logMethodCreation: result value].
	^result value!

compilerError: error
	"Display the compiler error errorString in the default textWidget."

	| widget text errorPos |

	(widget := sourceText) == nil ifTrue: [^nil].
	errorPos := error startPosition - 1 + widget getSelectionPosition x max: 0.
	widget
		setInputFocus ;
		insertAndShow: errorPos value: error message;
		setSelection: errorPos @ (errorPos + error message size)!

confirmClose

	^confirmClose
!

confirmClose: aBoolean

	confirmClose := aBoolean.
	self
		updateSaveMenu ;
		updateRevertMenu.
!

contentsOfTextWidget
	"Answer the current contents of the source code text widget."

	^sourceText getString!

copyAspectClass: aClass newName: aString
	"Rename an Aspect class."

	| newName classMapping methodsNotCopied references result |

	classMapping := Dictionary new.
	methodsNotCopied := Dictionary new.

	newName := aString asSymbol.
	(Smalltalk classAt: newName) notNil ifTrue: [
		System errorMessage: 'Cannot replace the aspect class named ', newName, ' because it already exists in the image.'.
		^false].
	classMapping at: aClass put: newName.

	classMapping notEmpty ifTrue: [
		self execShortOperation: [
			result := EtTools managerInterface
				copyClasses: classMapping
				in: aClass controller
				references: references
				methodsNotCopied: methodsNotCopied]].

	^result!

createAppIcon
	"Creates the application icon from a .ICO file"

	| format icons fileName |

	format := CgWinICOFileFormat new.
	fileName := self bitmapPath, 'aspect.ico'.
	icons := format loadFromFile: fileName.
	format hasErrorOccurred
		ifTrue: [ 
			Transcript cr; show: 'Could not load icon ', fileName.
			^CgIcon fromResource: 'oti'. ].

	^icons first.
!

createMenuBar: parent
	"Create the menu bar and the pulldown menus."

	| menuBar |

	menuBar := CwMenuBar new.
	menuBar addMenu: self menuFile.
	menuBar addMenu: self menuEdit.
	menuBar addMenu: self menuAspect.
	menuBar addMenu: self menuMethod.
	menuBar addMenu: self menuHelp.
	^menuBar.



			!

createPixmapButton: buttonClass for: aWidget pixmapName: aString
	"Create and answer a button of class buttonClass with a pixmap specified by aString.
	The button's parent is aWidget."

	| aButton colors palette buttonPixmap |

	aButton := buttonClass createWidget: aString parent: aWidget argBlock: nil.

	"Create a palette with the foreground and background colors
	of the button created, and the default palette.  This
	won't work for all cases, but it will work for most."
	colors := CgIndexedPalette default colors, (
	Array
		with: aButton backgroundColor
		with: aButton foregroundColor).
	palette := CgIndexedPalette colors: colors.
	aButton labelType: XmPIXMAP.
	 
	(buttonPixmap := self
		loadBitmapAsPixmap: aString
		foreground: (palette nearestPixelValue: aButton foregroundColor)
		background: (palette nearestPixelValue: aButton backgroundColor)
		palette: palette) isNil
		ifTrue: [Transcript cr; show: ('Could not load bitmap ' bindWith: aString)].
	buttonPixmap isNil
		ifFalse: [aButton labelPixmap: buttonPixmap].
	^aButton!

createWindow: parent
	"Private - comment"

	| form materialLabel matAspectLabel  aspectsLabel items 
	addAspectButton removeAspectButton radioBox |

	"Create form. Surrounds the whole window"
	form := parent 
		createForm: 'form'
		argBlock: [ :w |
			w
				verticalSpacing: 4;
				horizontalSpacing: 4 ].
	form addCallback: XmNdestroyCallback
		receiver: self
		selector: #destroy:clientData:callData:
		clientData: nil.
	form manageChild.

	materialLabel := form
		createLabel: 'materialsLabel'
		argBlock: nil.
	materialLabel labelString: 'Material classes:'.
	materialLabel manageChild.

	"Create material classes list"	
	items := self materialClasses.
	materialList := form 
		createScrolledList: 'materialList'
		argBlock: [ :w |
			w
				selectionPolicy: XmBROWSESELECT;
				visibleItemCount: 8;
				items: items ].
	materialList 
		addCallback: XmNbrowseSelectionCallback
		receiver: self
		selector: #materialSelected:clientData:callData:
		clientData: nil.
	materialList
		addCallback: XmNdefaultActionCallback
		receiver: self
		selector: #materialDoubleSelect:clientData:callData:
		clientData: nil.
	materialList manageChild.

	matAspectLabel := form
		createLabel: 'matAspectLabel'
		argBlock: nil.
	matAspectLabel labelString: 'Aspects:'.
	matAspectLabel manageChild.

	"Create material aspects list"
	matAspectList := form 
		createScrolledList: 'matAspectList'
		argBlock: [ :w |
			w
				selectionPolicy: XmBROWSESELECT;
				visibleItemCount: 8 ].
	matAspectList 
		addCallback: XmNbrowseSelectionCallback
		receiver: self
		selector: #materialAspectSelected:clientData:callData:
		clientData: nil.
	matAspectList
		addCallback: XmNdefaultActionCallback
		receiver: self
		selector: #matAspectDoubleSelect:clientData:callData:
		clientData: nil.
	matAspectList menu: self materialAspectMenu owner: self.
	matAspectList manageChild.

	"Create arrow buttons"
	addAspectButton := self 
		createPixmapButton: CwPushButton 
		for: form
		pixmapName: 'arrow_l'.
	addButtonPixmap := addAspectButton labelPixmap.
	addAspectButton 
		addCallback: XmNactivateCallback
		receiver: self
		selector: #addAspectButtonPressed:clientData:callData:
		clientData: nil.
	addAspectButton manageChild.

	removeAspectButton := self 
		createPixmapButton: CwPushButton 
		for: form
		pixmapName: 'arrow_r'.
	removeButtonPixmap := removeAspectButton labelPixmap.
	removeAspectButton 
		addCallback: XmNactivateCallback
		receiver: self
		selector: #removeAspectButtonPressed:clientData:callData:
		clientData: nil.
	removeAspectButton manageChild.

	"Create radio buttons"
	radioBox := form
		createRadioBox: 'radioBox'
		argBlock: [ :w |
			w orientation: XmHORIZONTAL ] .
	radioBox manageChild.

	materialMethodsButton := radioBox
		createToggleButton: 'Material aspect methods'
		argBlock: [ :w |
			w set: false ].
	materialMethodsButton
		addCallback: XmNvalueChangedCallback
		receiver: self
		selector: #materialMethodsPressed:clientData:callData:
		clientData: nil.
	materialMethodsButton manageChild.

	aspectMethodsButton := radioBox
		createToggleButton: 'Library aspect methods'
		argBlock: [ :w |
			w set: true ].
	aspectMethodsButton
		addCallback: XmNvalueChangedCallback
		receiver: self
		selector: #aspectMethodsPressed:clientData:callData:
		clientData: nil.
	aspectMethodsButton manageChild.

	"Create methods list"
	methodsList := CwObjectList
		createScrolledManagedWidget: 'methodsList'
			parent: form
			argBlock: [ :w |
				w
					scrollHorizontal: true;
					selectionPolicy: XmBROWSESELECT;
					visibleItemCount: 8 ].
	methodsList 
		addCallback: XmNbrowseSelectionCallback
		receiver: self
		selector: #methodSelected:clientData:callData:
		clientData: nil.
	methodsList
		addCallback: XmNmodifyVerifyCallback
		receiver: self
		selector: #selectionChangeVerify:clientData:callData:
		clientData: nil.
	methodsList manageChild.
	methodsList menu: self menuMethod owner: self.

	aspectsLabel := form
		createLabel: 'aspectsLabel'
		argBlock: nil.
	aspectsLabel labelString: 'Library Aspects:'.
	aspectsLabel manageChild.

	"Create aspects list"
	items := self aspectClasses.
	aspectsList := CwObjectList
		createScrolledManagedWidget: 'aspectsList'
			parent: form
			argBlock: [ :w |
				w
					scrollHorizontal: true;
					selectionPolicy: XmBROWSESELECT;
					visibleItemCount: 8;
					items: items ].
	aspectsList 
		addCallback: XmNbrowseSelectionCallback
		receiver: self
		selector: #aspectSelected:clientData:callData:
		clientData: nil.
	aspectsList
		addCallback: XmNdefaultActionCallback
		receiver: self
		selector: #aspectDoubleSelect:clientData:callData:
		clientData: nil.
	aspectsList
		addCallback: XmNmodifyVerifyCallback
		receiver: self
		selector: #selectionChangeVerify:clientData:callData:
		clientData: nil.
	aspectsList manageChild.
	aspectsList menu: self aspectMenu owner: self.

	"Create source code text widget"
	sourceText := form
		createScrolledText: 'sourceText'
		argBlock: [ :w |
			w
				editMode: XmMULTILINEEDIT;
				width: 400;
				height: 150;
				tabSpacing: EmTextCompatibilityFilter tabSize ].
	sourceText manageChild.
	(CwSmalltalkTextSelectionManager for: sourceText) connect.
	sourceText menu: self menuText owner: self.
	self addValueChangedCallback: sourceText.

	materialList setInputFocus.

	"Setting the widget attachements"
	materialLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHFORM ].

	materialList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: materialLabel;
			leftAttachment: XmATTACHFORM;
			rightAttachment: XmATTACHPOSITION;
			rightPosition: 31 ].

	matAspectLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: materialList ].

	matAspectList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: matAspectLabel;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: materialList;
			rightAttachment: XmATTACHPOSITION;
			rightPosition: 62 ].

	addAspectButton setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			topOffset: 70;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: matAspectList ].
	
	removeAspectButton setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: addAspectButton;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: matAspectList ].
			 
	aspectsLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: addAspectButton ].

	aspectsList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: aspectsLabel;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: addAspectButton;
			rightAttachment: XmATTACHFORM ].

	radioBox setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: materialList;
			leftAttachment: XmATTACHFORM ].

	methodsList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: radioBox;
			bottomAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHFORM;
			rightAttachment: XmATTACHPOSITION;
			rightPosition: 31 ].

	sourceText parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: radioBox;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: methodsList;
			rightAttachment: XmATTACHFORM;
			bottomAttachment: XmATTACHFORM ].
!

deleteAspectClass: aClass

	| result |

	self execShortOperation: [
		result := aClass deleteFrom: aClass controller.
		result ifTrue: [ self refreshAspects. ]].
	^result.
!

deleteMethod
	"Delete the selected method from their aspect and update
	 the list of methods."

	self changeRequest ifFalse: [^self].
	(System confirm: 'Delete selected method from the image?') ifTrue: [
		self execShortOperation: [
			self selectedAspect deleteMethod: self selectedMethod.
			self refreshMethods]]!

destroy: aRowColumn clientData: clientData callData: callData
	"Process the destroy callback."

	addButtonPixmap notNil 
		ifTrue: [ addButtonPixmap freePixmap ].
	removeButtonPixmap notNil 
		ifTrue: [ removeButtonPixmap freePixmap ].

	appIcon notNil
		ifTrue: [ appIcon freeIcon ].
!

enableAspectAddToMaterial
	"Answer whether the Aspect Add to Material menu option should be enabled."

	^(self selectedAspect notNil) and: [self selectedMaterial notNil].
!

enableAspectDelete
	"Answer whether the Aspect Delete menu option should be enabled."

	^self selectedAspect notNil.
!

enableAspectRemove
	"Answer whether the Aspect Remove from Material menu option should be enabled."

	^self selectedMaterialAspect notNil.
!

enableAspectRename
	"Answer whether the Aspect Rename menu option should be enabled."

	^self selectedAspect notNil.
!

enableAspectUpdate
	"Answer whether the Aspect Update Materials menu option should be enabled."

	^self selectedAspect notNil.
!

enableMethodDelete
	"Answer whether the Delete Method menu option should be enabled."

	^self selectedMethod notNil.

!

enableMethodNew
	"Answer whether the Method New Method Template menu option should be enabled."

	^(methodsSupplier = #Aspect) 
		ifTrue: [ self selectedAspect notNil ]
		ifFalse: [ self selectedMethod notNil ].
!

enableRevert
	"Answer whether the save menu option should be enabled."

	^self hasUnsavedChanges
!

enableSave
	"Answer whether the save menu option should be enabled."

	^self hasUnsavedChanges and: [
		(methodsSupplier = #Aspect) 
			ifTrue: [ true ]
			ifFalse: [ self selectedMethod notNil ]].


!

evaluateSelectionIn: textWidget ifFail: failBlock
	"Answer the result of evaluating the current selection in
	 the textWidget. If a problem is found, answer the result of
	 evaluating the failBlock instead."

	| text result failed errorPos |

	textWidget == nil ifTrue: [^failBlock value].
	(text := textWidget getSelection) == nil ifTrue: [^failBlock value]. 
	failed := false.
	self execShortOperation: [
		result := Compiler
			evaluate: text
			for: nil
			ifFail: [:error |
				errorPos := error startPosition - 1 + textWidget getSelectionPosition x max: 0.
				textWidget
					insertAndShow: errorPos value: error message;
					setSelection: errorPos @ (errorPos + error message size).
				failed := true]
	]. 
	failed ifTrue: [^failBlock value].
	System logEvaluate: text. 
	textWidget clearSelection. 
	^result!

execShortOperation: aBlock 
	"Evaluate aBlock in the UI process while showing a busy cursor in
	 the active shell."

	^System showBusyCursorWhile: aBlock
!

flushEvents
	"Dispatch events until there are no more."

	[CwAppContext default readAndDispatch] whileTrue: [].
!

hasUnsavedChanges
	"Answer true if the receiver has unsaved changes."

	^self confirmClose == true
!

hierarchyListDoubleSelect: widget
	selectedItem: selectedItem
	childrenSelector: childrenSelector
	asStringSelector: asStringSelector
	callData: callData

	"An entry in a hierarchical list box was double clicked. Expand or contract	the entry.	"

	| listPos listEntry indentionLevel newIndentionString children |

	listPos := callData itemPosition.
	listEntry := callData item.
	indentionLevel := self indentionLevel: listEntry.
	newIndentionString := ''.
	((indentionLevel + 1) * self indentionSize) timesRepeat: [
		newIndentionString := newIndentionString, ' ' ].

	(selectedItem perform: childrenSelector) notEmpty ifTrue: [
		"If the list box entry ends with '...' => the entry is not expanded"
		(listEntry copyFrom: listEntry size - 2 to: listEntry size) = '...' 
			ifTrue: [
				"Remove the dots '...' from the list box entry to show that it is expanded"
				widget 
					replaceItemsPos: (Array with: (listEntry copyWithout: $.))
					position: listPos.

				"Expand item. Show the children in the list box"
				children := selectedItem perform: childrenSelector.
				children := (children collect: [ :item | 
					newIndentionString, (item perform: asStringSelector), ( (item perform: childrenSelector) notEmpty
						ifTrue: ['...'] 
						ifFalse: ['']) ]) asSortedCollection.
				widget 
					addItems: children 
					position: listPos + 1.
			]
			ifFalse: [
				"Append dots '...' to the selected entry, to show that it is contracted"
				widget 
					replaceItemsPos: (Array with: listEntry, '...')
					position: listPos.

				"Contract item. Remove the entries for the children from the list box. 
				The child entries can themselves be expanded!! Their children must
				also be removed from the list box"
				[ (widget itemCount > listPos) 
					and: [
						(self indentionLevel: (widget items at: listPos + 1)) > indentionLevel ] ]
							whileTrue: [
								widget deletePos: listPos + 1].
			].
	].				
!

indentionLevel: aString
	"Calculates the indention level of the list box entry in aString"

	^(aString size - aString trimBlanks size) // self indentionSize.!

indentionSize
	"Returns the number of spaces, base aspects are indented
	in the aspects list box"

	^3.
!

initialize

	methodsSupplier := #Aspect.
!

loadBitmapAsPixmap: aString
	foreground: foreground
	background: background
	palette: palette
	"Private - Load the bitmap by name aString by contatenating BitmapPath to aString.
	 Reply the bitmap as a pixmap, or nil if the load failed.
	 The given foreground and background pixel values are used to expand
	 the bitmap to a pixmap of the same depth as the shell."

	| fileName bitmapReturn
	  widthReturn heightReturn xHotReturn yHotReturn
	  width height aBitmap aPixmap gc |

	bitmapReturn := ReturnParameter new.
	widthReturn := ReturnParameter new.
	heightReturn := ReturnParameter new.
	xHotReturn :=  yHotReturn := ReturnParameter null.

	fileName := self bitmapPath, aString.

	(shell screen rootWindow readBitmapFile: fileName
		widthReturn: widthReturn
		heightReturn: heightReturn
		bitmapReturn: bitmapReturn
		xHotReturn: xHotReturn
		yHotReturn: yHotReturn) ~= BitmapSuccess
			ifTrue: [
				Transcript cr; show: ('Could not load bitmap file ' bindWith: fileName).
				^ nil ].
	aBitmap := bitmapReturn value.

	aBitmap isNil
		ifTrue: [^nil].
	aPixmap :=
		shell screen rootWindow
			createPixmap: (width := widthReturn value)
			height:  (height := heightReturn value)
			depth: shell depth.
	aPixmap setPalette: palette.
	gc :=
		aPixmap
			createGC: (GCForeground | GCBackground)
			values:
				(CgGCValues new
					foreground: foreground;
					background: background).
	aBitmap
		copyPlane: aPixmap
		gc: gc
		srcX: 0
		srcY: 0
		width: width
		height: height
		destX: 0
		destY: 0
		plane: 1.
	aBitmap freePixmap.
	gc freeGC.
	^aPixmap
!

matAspectDoubleSelect: widget clientData: clientData callData: callData
	"An entry in the Material-Aspects list box was double clicked."

	self 
		hierarchyListDoubleSelect: widget
		selectedItem: self selectedMaterialAspect
		childrenSelector: #baseAspectClasses
		asStringSelector: #printString
		callData: callData
!

material: materialClass aspectMethods: aspectClass

	| methods |

	methods := materialClass aspectMethods: aspectClass.
	methods notNil 
		ifTrue: [
			methods := methods collect: [ :m | m selector asString ].
			^methods asSortedCollection]
		ifFalse: [ ^nil ].
!

materialAspectMenu
	"Answer a menu of operations for the material aspects widget."

	^CwMenu new
		title: 'Material Aspect';
		subMenuSelector: #menuMaterialAspect;
		add: #menuAspectRemoveFromMaterial    label: 'Re~move from Material' enable: [self enableAspectRemove] for: nil;
		add: #menuAspectUpdateLibrary    label: 'U~pdate Library Aspect...' enable: [self enableAspectRemove] for: nil;
		yourself
!

materialAspects: materialClass
	"Returns all aspects, the material has.
	Composit aspects are denoted with '...' " 

	| aspects |

	aspects := materialClass aspectClasses.
	aspects := aspects collect: [ :aspect |
		aspect name asString, (aspect isSimple
			ifFalse: ['...'] 
			ifTrue: ['']) ].
	^aspects asSortedCollection.
!

materialAspectSelected: widget clientData: clientData callData: callData
	"An entry in the Material Aspects list box was selected.
	Put the method selectors of the selected aspect in the Methods list box."

	(methodsSupplier = #Material) 
		ifTrue: [
			self refreshMethods.
			methodsList deselectAllItems.
			self changeContentsOf: sourceText to: ''. ].
!

materialClasses

	| materials |

	materials := Material subclasses.
	materials := materials collect: [ :cl |
		cl name asString, (cl subclasses isEmpty
			ifFalse: ['...'] 
			ifTrue: ['']) ].
	^materials asSortedCollection.
!

materialDoubleSelect: widget clientData: clientData callData: callData
	"An entry in the Materials list box was double clicked."

	self 
		hierarchyListDoubleSelect: widget
		selectedItem: self selectedMaterial
		childrenSelector: #subclasses
		asStringSelector: #printString
		callData: callData
!

materialMethodsPressed: widget clientData: clientData callData: callData
	"The Material aspect methods button has been pressed."

	callData set
		ifTrue: [
			self changeRequest ifFalse: [
				widget set: false.
				aspectMethodsButton set: true.
				^self].
			methodsSupplier := #Material.
			self refreshMethods.
			methodsList deselectAllItems.
			self changeContentsOf: sourceText to: ''. ].
!

materialSelected: widget clientData: clientData callData: callData
	"An entry in the list box Material classes was selected.
	Put the aspects of the selected material class in the Material aspects list box."

	self refreshMaterialAspects.
	(methodsSupplier = #Material) 
		ifTrue: [
			self refreshMethods.
			methodsList deselectAllItems.
			self changeContentsOf: sourceText to: ''. ].
!

menuAspect
	"Answer a menu of operations on aspects."

	aspectMenu == nil ifFalse: [^aspectMenu].
	^aspectMenu := CwMenu new
		title: '~Aspect';
		subMenuSelector: #menuAspect;
		add: #menuAspectNew  label: '~New...' enable: true for: nil;
		add: #menuAspectNewComposite      label: 'New ~Composite...' enable: true for: nil;
		add: #menuAspectDelete          label: '~Delete...'  enable: [self enableAspectDelete] for: nil;
		add: #menuAspectRename    label: '~Rename...'  enable: [self enableAspectRename] for: nil;
		addLine;
		add: #menuAspectAddToMaterial     	label: '~Add to Material' enable: [self enableAspectAddToMaterial] for: nil;
		add: #menuAspectUpdateMaterials 	label: '~Update Materials...' enable: [self enableAspectUpdate] for: nil;
		addLine;
		add: #menuAspectRemoveFromMaterial    label: 'Re~move from Material' enable: [self enableAspectRemove] for: nil;
		add: #menuAspectUpdateLibrary    label: 'U~pdate Library Aspect...' enable: [self enableAspectRemove] for: nil;
		yourself
!

menuAspectAddToMaterial

	self addAspectToMaterial.!

menuAspectDelete

	| aspect prompter materialsToUpdate containers result |

	self changeRequest ifFalse: [^self].
	aspect := self selectedAspect.

	"Test, if the aspect is contained in another aspect as a base aspect."
	"If so, the aspect can not be deleted."
	containers := CompositeAspect subclasses select: [ :cl | cl hasBaseAspect: aspect ].
	containers notEmpty ifTrue: [
		System message: 'The aspect ', aspect printString, 
			' is a base aspect in one ore more other aspects. ',
			'Therefor, it can not be deleted. Delete the container aspects first.'.
		prompter := CwListPrompter new.
		prompter title: 'Information'.
		prompter messageString: 'The following aspects contain the aspect ', aspect printString,
			' as a base aspect:'.
		prompter items: (containers collect: [ :cl | cl printString ]).
		prompter prompt.
		^self ]. 

	(System confirm: 'Delete the aspect ', aspect printString, '?') 
		ifTrue: [
			"Show the affected material classes"
			materialsToUpdate := Material materialsContaining: aspect.
			materialsToUpdate notEmpty ifTrue: [
				prompter := CwListPrompter new.
				prompter items: (materialsToUpdate collect: [ :material | material printString ]);
				selectedItems: prompter items first;
				messageString: 'Delete the aspect ', aspect printString, ' from following materials:';
				title: 'Please confirm'.
				prompter prompt isNil
					ifTrue: [ ^self ]
					ifFalse: [
						"Remove the aspect from the material classes"
						materialsToUpdate do: [ :material |
							(material removeAspectClass: aspect) ifFalse: [
								System message: 'Cannot remove aspect from material who does not have this aspect.' ]].
						self refreshMaterialAspects. ]].

			"Delete the aspect class"
			self execShortOperation: [
				aspect isSimple ifFalse: [
					aspect baseAspectClasses do: [ :cl |
						aspect removeBaseAspectClass: cl. ]]].
			self deleteAspectClass: aspect ].
!

menuAspectNew

	| newAspect |

	self changeRequest ifFalse: [^self].
	newAspect := AspectPrompter new prompt.
	newAspect notNil
		ifTrue: [ self refreshAspects. ].!

menuAspectNewComposite

	| newAspect |

	self changeRequest ifFalse: [^self].
	newAspect := CompositeAspectPrompter new prompt.
	newAspect notNil
		ifTrue: [ self refreshAspects. ].!

menuAspectRemoveFromMaterial

	self removeAspectFromMaterial.
!

menuAspectRename

	self renameAspect.
!

menuAspectUpdateLibrary
	"Compare the aspect in the material class with the aspect class
	and report the differences."

	| prompter differences updateDifferences |

	differences := AspectDifferences 
		differencesInAspect: self selectedMaterialAspect 
		Material: self selectedMaterial.

	(differences materialOnlyMethods isEmpty) & (differences aspectOnlyMethods isEmpty)
		ifTrue: [
			System message: 'There are no differences between the material and the library aspect.'.
			^self ].

	prompter := LibraryUpdatePrompter new.
	prompter differences: differences.
	updateDifferences := prompter prompt.

	"Synchronize the aspect in the aspect class with the aspect in the material class."
	updateDifferences notNil ifTrue: [
		(updateDifferences materialOnlyMethods isEmpty) & (updateDifferences aspectOnlyMethods isEmpty)
			ifFalse: [
				updateDifferences materialOnlyMethods do: [ :aCM |
					differences materialClass copyMethod: aCM To: differences aspectClass. ].
				updateDifferences aspectOnlyMethods do: [ :aCM |
					differences aspectClass deleteMethod: aCM. ].
				self refreshMethods. 
				System message: 'The library aspect has changed. You should update the material classes.' ]].
!

menuAspectUpdateMaterials

	| aspect prompter materialsToUpdate |

	self changeRequest ifFalse: [^self].
	aspect := self selectedAspect.
	
	(Material materialsContaining: aspect) notEmpty 
		ifTrue: [
			prompter := MaterialUpdatePrompter new.
			prompter changedAspect: aspect.
			materialsToUpdate := prompter prompt. 
			materialsToUpdate notNil ifTrue: [
				self execShortOperation: [
					materialsToUpdate do: [ :material |
						aspect synchronizeMaterial: (Smalltalk classAt: material asSymbol) . ]]]]
		ifFalse: [
			System message: 'The aspect ', aspect printString, ' is not contained in any material.' ].
 
!

menuEdit
	"Answer a pull down menu of operations for the text widget."

	^CwMenu new
		title: '~Edit';
		subMenuSelector: #menuEdit;
		add: #menuEditCut  label: 'Cu~t'        enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKx)
			acceleratorText: 'Shift+Del' ;
		add: #menuEditCopy label: '~Copy'      enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKc)
			acceleratorText: 'Ctrl+Ins' ;
		add: #menuEditPaste         label: '~Paste'     enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKv)
			acceleratorText: 'Shift+Ins' ;
		addLine;
		add: #menuEditExecute          label: '~Execute'	enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKe)
			acceleratorText: 'Ctrl+E' ;
		add: #menuEditDisplay        label: '~Display'	enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKd)
			acceleratorText: 'Ctrl+D' ;
		add: #menuEditInspect     label: '~Inspect' enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKq)
			acceleratorText: 'Ctrl+Q' ;
		addLine;
		add: #menuEditSelectAll     label: 'Select ~All' enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKa)
			acceleratorText: 'Ctrl+A' ;
		add: #menuEditSearchReplace     label: 'Search/Replace' enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKf)
			acceleratorText: 'Ctrl+F' ;
		yourself!

menuEditCopy
	"Process copy selection from edit menu."

	sourceText copySelection!

menuEditCut
	"Process cut selection from edit menu."

	sourceText cutSelection!

menuEditDisplay
	"Evaluate the current selection in the source code text widget
	 and display the result."

	| widget result position failed |

	widget := sourceText.
	position := widget getSelectionPosition.
	failed := false. 
	self execShortOperation: [
		result := self evaluateSelectionIn: widget ifFail: [failed := true]].
	failed  ifFalse: [
		result := result printString.
		widget
			setInsertionPosition: position y ;
			insertAndShow: position y value: ' ', result;
			setSelection: position y @ widget getInsertionPosition]!

menuEditExecute
	"Evaluate the current selection in the source code text widget."

	self execShortOperation: [
		self evaluateSelectionIn: sourceText ifFail: []]
!

menuEditInspect
	"Inspect the result of evaluating the current selection in the
	source code text widget."

	| result failed |

	failed := false. 
	self execShortOperation: [
		result := self evaluateSelectionIn: sourceText ifFail: [failed := true].
		failed ifFalse: [result inspect]]!

menuEditPaste
	"Process paste selection from edit menu."

	sourceText
		setInputFocus;
		paste!

menuEditSearchReplace
	"Open a modeless search replace dialog. We allow the user to have multiple
	 search replace dialogs open at once."

	self execShortOperation: [
		"On some platforms the selection will not show unless the widget has focus."
		sourceText setInputFocus.
		(CwTextSearchDialog for: sourceText)
			mwmInputMode: MWMINPUTMODELESS;
			open]!

menuEditSelectAll
	"Select all of the text in the source code text widget."

	"On some platforms the selection will not show unless the widget has focus."
	sourceText setInputFocus.

	"Some Windows perform processing when the text widget gets focus,
	 and in some cases this may interact with the selection.  We must
	 make sure the callbacks have been sent before setting the new selection."
	self flushEvents.

	sourceText setSelection: 0 @ (sourceText getLastPosition).
!

menuFile
	"Answer a menu of operations for file operations."

	fileMenu == nil ifFalse: [^fileMenu].
	^fileMenu := CwMenu new
		title: '~File';
		subMenuSelector: #menuFile;
		add: #menuFileNew  label: '~New' enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKn)
			acceleratorText: 'Ctrl+N' ;
		add: #menuFileOpen      label: '~Open...' enable: true for: nil
			accelerator: (CwAccelerator mask: ControlMask keysym: XKo)
			acceleratorText: 'Ctrl+O' ;
		add: #menuFileSave          label: '~Save'          enable: [self enableSave] for: nil	"update later"
			accelerator: (CwAccelerator mask: ControlMask keysym: XKs)
			acceleratorText: 'Ctrl+S' ;
		add: #menuFileRevert       label: '~Revert'  enable: [self enableRevert] for: nil		"update later"
			accelerator: (CwAccelerator mask: ControlMask keysym: XKr)
			acceleratorText: 'Ctrl+R' ;
		addLine;
		add: #menuFileUpdateLists     	label: '~Update Lists';
		addLine;
		add: #menuFileSaveImage     	label: 'Save Image...';
		add: #menuFileSaveImageNamed    label: 'Save Image As...';
		add: #menuFileExitSmalltalk 	label: 'Exit ',System startUpClass applicationName;
		yourself!

menuFileExitSmalltalk
	"Exit the image."

	| result cwd |

	result := ((CwMessagePrompter for: CwWMShell focusShell)	"safety measure so this can be invoked if receiver shell is invalid"
		iconType: XmICONWARNING ;
		buttonType: XmYESNOCANCEL ;
		messageString: ('About to exit ',System startUpClass applicationName,'.\Save the image to file "') addLineDelimiters, System imageFileName, '"?' ;
		prompt).

	result isNil ifTrue:[^self].
	result ifTrue: [self execShortOperation: [
		cwd := CfsDirectoryDescriptor getcwd.
		CfsDirectoryDescriptor chdir: CfsDirectoryDescriptor startUpDirectory. 
		System saveImage.
		CfsDirectoryDescriptor chdir: cwd. 
	]].
	System exit.
!

menuFileNew
	"Open a new workspace."

	self execShortOperation: [
		EtWorkspace new open]
!

menuFileOpen
	"Open a new workspace on the contents of a file selected by
	 the user."

	| fileName |

	self execShortOperation: [
		(fileName := self promptForFileName: 'Open File') notNil ifTrue: [
		(EtWorkspace forFileNamed: fileName) open]]
!

menuFileRevert
	"Restore the original text for the widget which receives the generic
	 menu commands."

	self revertToSaved.
!

menuFileSave
	"Save the contents of the source code text widget."

	self saveChanges.
!

menuFileSaveImage
	"Save the image."
	
	| cwd | 

	self execShortOperation: [
		(System confirm: ('Save the image to file "') addLineDelimiters, System imageFileName, '"?')
			ifTrue: [
				cwd := CfsDirectoryDescriptor getcwd.
				CfsDirectoryDescriptor chdir: CfsDirectoryDescriptor startUpDirectory. 
				System saveImage.
				CfsDirectoryDescriptor chdir: cwd. ]].
!

menuFileSaveImageNamed
	"Save the image but prompt for fileName"

	| fileName cwd |

	self execShortOperation: [
		(fileName := self promptForFileName: 'File Name for Image ?' defaultName: System imageFileName) notNil ifTrue: [
			cwd := CfsDirectoryDescriptor getcwd.
			CfsDirectoryDescriptor chdir: CfsDirectoryDescriptor startUpDirectory. 
			((CfsStat stat: fileName) isCfsError or: [
				(System confirm: 'The file ', fileName, ' already exists.\Save image over it?' addLineDelimiters)])
					ifTrue: [System showBusyCursorWhile: [System saveImage: fileName]].
			CfsDirectoryDescriptor chdir: cwd. ]].
!

menuFileUpdateLists
	"Update the materials and aspects lists."

	self changeRequest ifFalse: [^self].
	self execShortOperation: [
		self refreshMaterials.
		self refreshMaterialAspects.
		self refreshAspects.
		self refreshMethods.
		self changeContentsOf: sourceText to: ''. ].
!

menuHelp
	"Answer a menu of help operations."

	^CwMenu new
		title: '~Help';
		subMenuSelector: #menuHelp;
		add: #menuHelpAbout  label: '~About Aspect Browser...' enable: true for: nil;
		yourself
!

menuHelpAbout
	"Open a new workspace on the contents of a file selected by
	 the user."

	AboutAspectBrowserPrompter new prompt.!

menuMethod
	"Answer a menu of operations on methods."

	methodMenu == nil ifFalse: [^methodMenu].
	^methodMenu := CwMenu new
		title: '~Method';
		subMenuSelector: #menuMethod;
		add: #menuMethodNew  label: '~New Method Template' enable: [self enableMethodNew] for: nil;
		add: #menuMethodDelete      label: '~Delete Method' enable: [self enableMethodDelete] for: nil;
		yourself
!

menuMethodDelete

	self deleteMethod.
	methodsList deselectAllItems.
	self changeContentsOf: sourceText to: ''.!

menuMethodNew

	self newMethod.
!

menuText
	"Answer a menu of operations for the source code widget."

	| receiver |
	textMenu == nil ifFalse: [^textMenu].
	receiver := sourceText.
	^textMenu := CwMenu new
		title: '~Edit';
		subMenuSelector: #textMenu;
		add: #menuFileRevert       		label: '~Revert'	enable: [self enableRevert];	"update later"
		addLine;
		add: #cutSelection  label: 'Cu~t'        enable: true for: receiver;
		add: #copySelection label: '~Copy'      enable: true for: receiver;
		add: #paste         label: '~Paste'     enable: true for: receiver;
		addLine;
		add: #menuEditExecute			label: '~Execute';
		add: #menuEditDisplay			label: '~Display';
		add: #menuEditInspect			label: '~Inspect';
		addLine;
		add: #menuFileSave          		label: '~Save'      enable: [self enableSave];	"update later"
		yourself!

methodSelected: widget clientData: clientData callData: callData
	"An entry in the Methods list box was selected.
	Write the source code of the selected method in the source code edit field"

	| method |

	method := self selectedMethod.
	method notNil ifTrue: [
		self changeContentsOf: sourceText to: (self sourceStringFor: method) ].
!

newMethod
	"Change to showing a new method template."

	self changeRequest ifFalse: [^self].
	self execShortOperation: [
		self changeContentsOf: sourceText to: EtTools publicTemplate. 
		self setTextChangeFlag: sourceText ].!

promptForFileName: prompt
	"Answer a file name entered by the user."

	^self promptForFileName: prompt defaultName: nil
!

promptForFileName: prompt defaultName: fileName
	"Answer a file name entered by the user."

	| prompter fileNameStart directory file | 
	prompter := (CwFileSelectionPrompter for: shell).
	prompter title: prompt.
	fileName notNil ifTrue: [ 	
		fileNameStart := fileName findLast: [ :ch | ch = CfsDirectoryDescriptor pathSeparator].
		fileNameStart > 0 ifTrue: [ 
				directory := (fileName copyFrom: 1 to: fileNameStart - 1) trimSeparators.
				directory notEmpty ifTrue: [ prompter searchPath: directory].
				file := (fileName copyFrom: fileNameStart  + 1 to: fileName size) trimSeparators.
				file notEmpty ifTrue: [ prompter fileName: file].
		] ifFalse: [
				file := fileName trimSeparators.
				file notEmpty ifTrue: [ prompter fileName: file]. 
		]
	].
	^ prompter prompt!

refreshAspects

	| aspects |

	aspectsList deselectAllItems.
	aspectsList deleteAllItems.
	aspects := self aspectClasses.
	aspects notNil ifTrue: [
		aspectsList items: aspects ].

	(methodsSupplier = #Aspect) 
		ifTrue: [
			self refreshMethods.
			self changeContentsOf: sourceText to: ''. ].
!

refreshMaterialAspects

	| material aspects |

	matAspectList deselectAllItems.
	matAspectList deleteAllItems.
	material := self selectedMaterial.
	material notNil ifTrue: [
		aspects := self materialAspects: material.
		aspects notNil ifTrue: [
			matAspectList items: aspects ]].
!

refreshMaterials

	| materials |

	materials := self materialClasses.
	materialList deselectAllItems.
	materialList deleteAllItems.
	materials notNil ifTrue: [
		materialList items: materials ].
!

refreshMethods

	| methods aspectClass materialClass |

	methodsList deselectAllItems.
	methodsList deleteAllItems.
	(methodsSupplier = #Aspect) 
		ifTrue: [
			aspectClass := self selectedAspect.
			aspectClass notNil ifTrue: [
				methods := self aspectMethods: aspectClass. ]]
		ifFalse: [		"methodsSupplier = #Material"
			materialClass := 	self selectedMaterial.
			aspectClass := self selectedMaterialAspect.
			(materialClass notNil & aspectClass notNil) ifTrue: [
				methods := self 
					material: materialClass
					aspectMethods: aspectClass. ]].

	methods notNil ifTrue: [
		methodsList items: methods ].!

removeAspectButtonPressed: widget clientData: clientData callData: callData
	"The Remove Aspect button has been pressed."

	self removeAspectFromMaterial.
!

removeAspectFromMaterial

	| materialAspect result |

	materialAspect := self selectedMaterialAspect.
	materialAspect notNil 
		ifTrue: [
			result := self selectedMaterial removeAspectClass: materialAspect.
			result 
				ifTrue: [
					self refreshMaterialAspects.
					self refreshMethods.
					self changeContentsOf: sourceText to: ''. ]
				ifFalse: [
					System message: 'Cannot remove aspect from material who does not have this aspect.' ]]
		ifFalse: [ System message: 'No aspect selected.' ].
!

removeValueChangedCallback: widget 
	"Remove the valueChangedCallback."

	widget 
		removeCallback: XmNvalueChangedCallback
		receiver: self
		selector: #valueChanged:clientData:callData:
		clientData: nil.
	self confirmClose: true!

renameAspect
	"Rename the selected Aspect class."

	| aspectClass newName oldName baseAspects newAspectClass result |

	aspectClass := self selectedAspect.
	oldName := aspectClass name asString.
	newName := System prompt: 'New name for aspect class?' answer: aspectClass name.
	newName isNil ifTrue: [^false].

	"Rename the method category."
	(aspectClass categoryExists: oldName) ifTrue: [
		result := aspectClass 
			renameCategory: oldName
			to: newName.
		result ifFalse: [ ^false ]].

	"Copy the aspect class."
	result := self copyAspectClass: aspectClass newName: newName.
	newAspectClass := Smalltalk classAt: newName asSymbol.
	(newAspectClass isNil or: [result not]) ifTrue: [
		"Rollback category name change"
		(aspectClass categoryExists: newName) ifTrue: [
			aspectClass 
				renameCategory: newName
				to: oldName ].
		^false ].

	"Copy the value of the class variable BaseAspects to the new class."
	aspectClass isSimple ifFalse: [
		baseAspects := aspectClass baseAspects.
		newAspectClass baseAspects: baseAspects ].

	"Rename the aspect in the composite aspect classes."
	CompositeAspect subclasses do: [ :aspect |
		(aspect baseAspectClasses includes: aspectClass) ifTrue: [
			aspect removeBaseAspectClass: aspectClass.
			aspect addBaseAspectClass: newAspectClass ]].

	"Rename the aspect in the material classes."
	Material allSubclasses do: [ :material |
		(material aspects includes: aspectClass name) ifTrue: [
			material removeAspectClass: aspectClass.
			material addAspectClass: newAspectClass.
			(material categoryExists: oldName) ifTrue: [
				material 
					renameCategory: oldName
					to: newName ]]].

	"Delete the old aspect class"
	self deleteAspectClass: aspectClass.
	self refreshMaterialAspects.
	self refreshMethods.!

resetTextChangeFlag: widget  
	"Reset the text change flag to indicate the textWidget is not changed and hook callbacks if
	required. "

	self confirmClose = true ifTrue: [self addValueChangedCallback: widget	].
	self confirmClose: false. !

revertToSaved
	"Restore the original text for the widget which receives the generic
	 menu commands."

	| method text |
	
	method := self selectedMethod.
	method notNil
		ifTrue: [ text := self sourceStringFor: method ]
		ifFalse: [ text := '' ].

	self execShortOperation: [
		self
			changeContentsOf: sourceText
			to: text ].
!

ringBell
	"Private - Ring the bell."

	CgDisplay default bell: 10.!

saveChanges
	"Save the contents of the receiver.
	 Answer true if the changes were saved - otherwise false."

	| widget result |

	widget := sourceText.
	widget deferRedraw: [ 
		widget clearSelection.
		result := self acceptText: (self contentsOfTextWidget) from: self].
	result ifTrue: [ self resetTextChangeFlag: widget ].
	^result.
!

selectedAspect

	| aspects |

	aspects := aspectsList selectedItems.
	aspects isEmpty 
		ifFalse: [
			"Remove the dots '...' in case it is a ComplexAspect and 
			the spaces in case it is a base aspect"
			^Smalltalk classAt: (aspects first copyWithout: $.) trimBlanks asSymbol ]
		ifTrue: [ ^nil ].
!

selectedMaterial

	| materials |

	materials := materialList selectedItems.
	materials isEmpty 
		ifFalse: [
			"Remove the dots '...' in case it has subclasses and 
			the spaces in case it is a subclass"
			^Smalltalk classAt: (materials first copyWithout: $.) trimBlanks asSymbol ]
		ifTrue: [ ^nil ].
!

selectedMaterialAspect

	| aspects |

	aspects := matAspectList selectedItems.
	aspects isEmpty 
		ifFalse: [
			"Remove the dots '...' in case it is a ComplexAspect and 
			the spaces in case it is a base aspect"
			^Smalltalk classAt: (aspects first copyWithout: $.) trimBlanks asSymbol ]
		ifTrue: [ ^nil ].
!

selectedMethod

	| supplierClass methods |

	(methodsSupplier = #Aspect) 
		ifTrue: [ supplierClass := self selectedAspect. ]
		ifFalse: [ supplierClass := self selectedMaterial. ].
				
	supplierClass notNil 
		ifTrue: [
			methods := methodsList selectedItems.
			methods isEmpty
				ifFalse: [ ^supplierClass methodDictionary at: methods first asSymbol. ]
				ifTrue: [ ^nil ]]
		ifFalse: [ ^nil ].!

selectionChangeVerify: w clientData: ignore callData: callData
	"Process a modifyVerifyCallback from a list widget."

	callData doit: self changeRequest.
!

setTextChangeFlag: widget  
	"Set the text change flag to indicate the textWidget is changed and remove 
	the callback since we do not need it until the text is reset to unmodified state."

	self confirmClose: true.
	self removeValueChangedCallback: widget.!

sourceStringFor: method

	"Private - Return the source code for the method."

	| source |
	^(source := method sourceString) isNil
		ifTrue: [method selector, ' can not access source code']
		ifFalse: [source]
!

updateRevertMenu
	"Update the enable state of the revert menu.
	 Since this menu has an accelerator, we must keep it up to date."

	| revert |

	(fileMenu == nil or: [textMenu == nil])
		ifTrue: ["handle mutation" ^self].
	fileMenu enable: #menuFileRevert if: (revert := self enableRevert).
	textMenu enable:#menuFileRevert if: revert.
!

updateSaveMenu
	"Update the enable state of the save menu.
	 Since this menu has an accelerator, we must keep it up to date."

	| save |

	(fileMenu == nil or: [textMenu == nil])
		ifTrue: ["handle mutation" ^self].
	fileMenu enable: #menuFileSave if: (save := self enableSave).
	textMenu enable:#menuFileSave if: save.
!

valueChanged: widget clientData: ignore1 callData: ignore2
	"Process the valueChangedCallback from the text widget."

	self setTextChangeFlag: widget.
! !


!AspectPrompter privateMethods !

createChildren
	"Create, initialize and manage the children of the bulletinBoardWidget."

	| bb nameLabel |

	super createChildren.

	bb := self bulletinBoardWidget.

	nameLabel := bb
		createLabel: 'nameLabel'
		argBlock: nil.
	nameLabel labelString: 'Name:'.
	nameLabel manageChild.

	nameText := bb
		createText: 'nameText'
		argBlock: nil.
	nameText manageChild.

	self focusWidget: nameText.

	"Set the widget attachments"
	nameLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			topOffset: 25;
			leftAttachment: XmATTACHFORM ].

	nameText setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			topOffset: 22;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: nameLabel;
			rightAttachment: XmATTACHWIDGET;
			rightWidget: buttonOK;
			rightOffset: 15 ].
!

ok: w clientData: clientData callData: callData
	"The OK button was pressed.
	Create a new aspect class with the entered name."

	| aspectName newAspect aspectClass |

	aspectName := nameText getString.
	(aspectName size > 0)
		ifFalse: [
			System message: 'Please, enter a name for the new aspect.' ]
		ifTrue: [
			"Test if a class with aspectName allready exists"
			(Smalltalk classAt: aspectName asSymbol) isNil
				ifFalse: [
					System message: 'Can not create new aspect. A class with name ', aspectName,
						' allready exists.'.
					^self. ].

			aspectClass := Smalltalk classAt: #SimpleAspect.
			aspectClass isNil
				ifTrue: [
					System message: 'Can not create new aspect. Base class SimpleAspect not found in the image.'.
					^self. ].

			"Create the new aspect class"
			newAspect := EtTools managerInterface
				createClass: aspectName asSymbol
				subclassOf: aspectClass
				type: #pointer
				in: aspectClass applications first.
			self result: newAspect.

			newAspect notNil
				ifFalse: [ ^self ].

			super ok: w clientData: clientData callData: callData. 
		].!

title
	"Answer the window title to use for the receiver."

	^'New Aspect'
! !


!CompositeAspectPrompter privateMethods !

aspectClasses
	"Returns all aspect classes directly derived from class Aspect.
	Composit aspects are denoted with '...' " 

	| aspects |

	aspects := (SimpleAspect subclasses collect: [ :cl | cl name asString ]) asSortedCollection.
	aspects addAll: (CompositeAspect subclasses collect: [ :cl | cl name asString ]).
	^aspects!

buttonLeft: widget clientData: clientData callData: callData
	"The left arrow button was pressed. Move the selected
	items from the new aspect list box to the Aspects list box."

	| aspects |

	aspects := newAspectList selectedItems.
	aspects isEmpty 
		ifFalse: [ 
			aspectsList 
				addItems: aspects
				position: 0.
			newAspectList deleteItems: aspects].
!

buttonRight: widget clientData: clientData callData: callData
	"The right arrow button was pressed. Move the selected
	items from the Aspects list box to the new aspect list box."

	| aspects |

	aspects := aspectsList selectedItems.
	aspects isEmpty 
		ifFalse: [ 
			newAspectList 
				addItems: aspects
				position: 0.
			aspectsList deleteItems: aspects].
!

createChildren
	"Create, initialize and manage the children of the bulletinBoardWidget."

	|  bb items nameLabel aspectsLabel newAspectLabel
	buttonRight buttonLeft |

	super createChildren.

	bb := self bulletinBoardWidget.

	nameLabel := bb
		createLabel: 'nameLabel'
		argBlock: nil.
	nameLabel labelString: 'Name:'.
	nameLabel manageChild.

	nameText := bb
		createText: 'nameText'
		argBlock: nil.
	nameText manageChild.

	aspectsLabel := bb
		createLabel: 'aspectsLabel'
		argBlock: nil.
	aspectsLabel labelString: 'Aspects:'.
	aspectsLabel manageChild.

	"Create aspect classes list"	
	items := self aspectClasses.
	aspectsList := bb
		createScrolledList: 'aspectsList'
		argBlock: [ :w |
			w
				selectionPolicy: XmMULTIPLESELECT;
				visibleItemCount: 8;
				items: items;
				width: 150 ].
	aspectsList manageChild.

	"Create the arrow buttons"
	buttonRight := bb
		createPushButton: ' > '
		argBlock: nil.
	buttonRight manageChild.
	buttonRight
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonRight:clientData:callData:
		clientData: nil.

	buttonLeft := bb
		createPushButton: ' < '
		argBlock: nil.
	buttonLeft manageChild.
	buttonLeft
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonLeft:clientData:callData:
		clientData: nil.

	newAspectLabel := bb
		createLabel: 'newAspectLabel'
		argBlock: nil.
	newAspectLabel labelString: 'New aspect:'.
	newAspectLabel manageChild.

	"Create new aspect list"	
	newAspectList := bb
		createScrolledList: 'newAspectList'
		argBlock: [ :w |
			w
				selectionPolicy: XmMULTIPLESELECT;
				visibleItemCount: 8;
				width: 150 ].
	newAspectList manageChild.

	self focusWidget: nameText.

	"Set the widget attachments"

	nameLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			topOffset: 13;
			leftAttachment: XmATTACHFORM ].

	nameText setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: nameLabel ].

	aspectsLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: nameLabel;
			topOffset: 25;
			leftAttachment: XmATTACHFORM ].

	aspectsList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: aspectsLabel;
			leftAttachment: XmATTACHFORM ].

	buttonRight setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHOPPOSITEWIDGET;
			topWidget: aspectsList;
			topOffset: 40;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: aspectsList ].

	buttonLeft setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: buttonRight;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: aspectsList ].

	newAspectLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: nameLabel;
			topOffset: 25;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: buttonRight ].

	newAspectList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: newAspectLabel;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: buttonRight;
			rightAttachment: XmATTACHWIDGET;
			rightWidget: buttonOK;
			rightOffset: 15 ].
!

ok: w clientData: clientData callData: callData
	"The OK button was pressed.
	Create a new composite aspect class and set the
	base aspects according to the selected items in 
	the New Aspect list box."

	| aspectName baseAspects newAspect aspectClass |

	aspectName := nameText getString.
	(aspectName size > 0)
		ifFalse: [
			System message: 'Please, enter a name for the new aspect.' ]
		ifTrue: [
			"Test if a class with aspectName allready exists"
			(Smalltalk classAt: aspectName asSymbol) isNil
				ifFalse: [
					System message: 'Can not create new aspect. A class with name ', aspectName,
						' allready exists.'.
					^self. ].

			baseAspects := newAspectList items.
			baseAspects isEmpty 
				ifTrue: [
					System message: 'Please, select one or more base aspects for the composite aspect.'.
					^self. ].

			aspectClass := Smalltalk classAt: #CompositeAspect.
			aspectClass isNil
				ifTrue: [
					System message: 'Can not create new aspect. Base class CompositeAspect not found in the image.'.
					^self. ].

			"Create the new aspect class"
			newAspect := EtTools managerInterface
				createClass: aspectName asSymbol
				subclassOf: aspectClass
				type: #pointer
				in: aspectClass applications first.
			self result: newAspect.

			newAspect notNil
				ifTrue: [
					baseAspects do: [ :aspect |
						newAspect addBaseAspectClass: (Smalltalk classAt: aspect asSymbol). ]]
				ifFalse: [ ^self ].

			super ok: w clientData: clientData callData: callData. 
		].!

title
	"Answer the window title to use for the receiver."

	^'New Composite Aspect'
! !


!CwStandardPrompter privateMethods !

bulletinBoardArgs
	"Answer an argBlock to use when creating the bulletinBoardWidget."

	^[:w |
		w
			marginWidth: 10;
			marginHeight: 10;
			verticalSpacing: 4;
			horizontalSpacing: 4;
			autoUnmanage: false;
			mappedWhenManaged: true ]
!

buttonCancel: w clientData: clientData callData: callData
	"Perform the callback processing for buttonCancel activate."

	self cancel: w clientData: clientData callData: callData.
!

buttonOk: w clientData: clientData callData: callData
	"Perform the callback processing for buttonOk activate."

	self ok: w clientData: clientData callData: callData
!

createBulletinBoard
	"Create the bulletinBoardWidget which is the immediate child of the shell.
	 This is a Form for instances of the receiver and subclasses."

	self bulletinBoardWidget: (self shell createForm: 'form' argBlock: self bulletinBoardArgs).!

createChildren
	"Create, initialize and manage the children of the bulletinBoardWidget.
	 Create the Ok und Cancel buttons at the upper, right corner of the prompter."

	|  bb |

	bb := self bulletinBoardWidget.
	buttonOK := bb
		createPushButton: 'OK'
		argBlock: [ :w |
			w width: 65 ].
	buttonOK manageChild.
	buttonOK
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonOk:clientData:callData:
		clientData: nil.
	bb defaultButton: buttonOK.

	buttonCancel := bb
		createPushButton: 'Cancel'
		argBlock: [ :w |
			w width: 65 ].
	buttonCancel manageChild.
	buttonCancel
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonCancel:clientData:callData:
		clientData: nil.

	buttonOK setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			rightAttachment: XmATTACHFORM ].

	buttonCancel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: buttonOK;
			rightAttachment: XmATTACHFORM ].
!

shellArgs
	"Answer an argBlock to use as the create args for the receiver's shell."

	^[:w | w
		title: self title;
		mwmDecorations: MWMDECORMENU | MWMDECORBORDER | MWMDECORTITLE ]! !


!LibraryUpdatePrompter publicMethods !

differences

	^differences.
!

differences: anObject

	differences := anObject.
	self aspectClass: differences aspectClass.
	self materialClass: differences materialClass.
! !

!LibraryUpdatePrompter privateMethods !

aspectClass

	^aspectClass.
!

aspectClass: aClass

	aspectClass := aClass.
!

buttonSelectAll: widget clientData: clientData callData: callData
	"The Select All button was pressed. Select all entries in
	the list boxes."

	materialList selectedItems: materialList items.
	aspectList selectedItems: aspectList items.
!

createChildren
	"Create, initialize and manage the children of the bulletinBoardWidget."

	|  bb items description1Label description2Label materialLabel aspectLabel buttonSelectAll |

	super createChildren.

	bb := self bulletinBoardWidget.

	description1Label := bb
		createLabel: 'description1Label'
		argBlock: nil.
	description1Label labelString: 'The list boxes show the differences between the two aspects.'.
	description1Label manageChild.

	description2Label := bb
		createLabel: 'description2Label'
		argBlock: nil.
	description2Label labelString: 'Select the differences, you want to update.'.
	description2Label manageChild.

	materialLabel := bb
		createLabel: 'materialLabel'
		argBlock: nil.
	materialLabel labelString: 'Material ', self materialClass printString, ':'.
	materialLabel manageChild.

	"Create material aspect method list"	
	items := self differences materialOnlyMethods collect: [ :cm | cm selector asString ].
	materialList := bb
		createScrolledList: 'materialList'
		argBlock: [ :w |
			w
				selectionPolicy: XmMULTIPLESELECT;
				visibleItemCount: 8;
				items: items;
				width: 200 ].
	materialList manageChild.

	aspectLabel := bb
		createLabel: 'aspectLabel'
		argBlock: nil.
	aspectLabel labelString: 'Library Aspect ', self aspectClass printString, ':'.
	aspectLabel manageChild.

	"Create library aspect method list"	
	items := self differences aspectOnlyMethods collect: [ :cm | cm selector asString ].
	aspectList := bb
		createScrolledList: 'aspectList'
		argBlock: [ :w |
			w
				selectionPolicy: XmMULTIPLESELECT;
				visibleItemCount: 8;
				items: items;
				width: 200 ].
	aspectList manageChild.

	buttonSelectAll := bb
		createPushButton: ' Select all '
		argBlock: nil.
	buttonSelectAll manageChild.
	buttonSelectAll
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonSelectAll:clientData:callData:
		clientData: nil.

	self focusWidget: materialList.

	"Set the widget attachments"

	description1Label setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHFORM ].

	description2Label setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: description1Label;
			topOffset: 0;
			leftAttachment: XmATTACHFORM ].

	materialLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: description2Label;
			leftAttachment: XmATTACHFORM ].

	materialList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: materialLabel;
			leftAttachment: XmATTACHFORM ].

	aspectLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: description2Label;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: materialList ].

	aspectList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: materialLabel;
			leftAttachment: XmATTACHWIDGET;
			leftWidget: materialList;
			rightAttachment: XmATTACHWIDGET;
			rightWidget: buttonSelectAll;
			rightOffset: 15 ].

	buttonSelectAll setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: buttonCancel;
			topOffset: 38;
			rightAttachment: XmATTACHFORM ].
!

materialClass

	^materialClass.
!

materialClass: aClass

	materialClass := aClass.
!

ok: w clientData: clientData callData: callData
	"The OK button was pressed. Return the selected
	Methods in the Material and Aspect list boxes in 
	a AspectDifferences object."

	| materialMethods aspectMethods resultDifferences |

	materialMethods := materialList selectedItems collect: [ :m | 
		self differences materialClass methodDictionary at: m asSymbol ].
	aspectMethods := aspectList selectedItems collect: [ :m | 
		self differences aspectClass methodDictionary at: m asSymbol ].

	resultDifferences := AspectDifferences new.
	resultDifferences aspectClass: differences aspectClass;
		materialClass: differences materialClass;
		aspectOnlyMethods: aspectMethods;
		materialOnlyMethods: materialMethods.

	self result: resultDifferences.
	super ok: w clientData: clientData callData: callData. 
!

title
	"Answer the window title to use for the receiver."

	^'Update Library Aspect'
! !


!MaterialUpdatePrompter publicMethods !

changedAspect

	^changedAspect.
!

changedAspect: anAspect

	changedAspect := anAspect.
! !

!MaterialUpdatePrompter privateMethods !

buttonSelectAll: widget clientData: clientData callData: callData
	"The Select All button was pressed. Select all entries in
	the materials list box."

	materialsList selectedItems: materialsList items.
!

createChildren
	"Create, initialize and manage the children of the bulletinBoardWidget."

	|  bb items nameLabel description1Label description2Label materialsLabel buttonSelectAll |

	super createChildren.

	bb := self bulletinBoardWidget.

	nameLabel := bb
		createLabel: 'nameLabel'
		argBlock: nil.
	nameLabel labelString: 'Aspect:   ', self changedAspect printString.
	nameLabel manageChild.

	description1Label := bb
		createLabel: 'description1Label'
		argBlock: nil.
	description1Label labelString: 'Select the material classes,'.
	description1Label manageChild.

	description2Label := bb
		createLabel: 'description2Label'
		argBlock: nil.
	description2Label labelString: 'you want to update.'.
	description2Label manageChild.

	materialsLabel := bb
		createLabel: 'materialsLabel'
		argBlock: nil.
	materialsLabel labelString: 'Affected material classes:'.
	materialsLabel manageChild.

	"Create material classes list"	
	items := Material materialsContaining: self changedAspect.
	items := (items collect: [ :cl | cl printString ]) asSortedCollection.
	materialsList := bb
		createScrolledList: 'materialsList'
		argBlock: [ :w |
			w
				selectionPolicy: XmMULTIPLESELECT;
				visibleItemCount: 8;
				items: items;
				selectedItems: items. ].
	materialsList manageChild.

	buttonSelectAll := bb
		createPushButton: ' Select all '
		argBlock: nil.
	buttonSelectAll manageChild.
	buttonSelectAll
		addCallback: XmNactivateCallback
		receiver: self
		selector: #buttonSelectAll:clientData:callData:
		clientData: nil.

	self focusWidget: materialsList.

	"Set the widget attachments"

	nameLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHFORM;
			leftAttachment: XmATTACHFORM ].

	description1Label setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: nameLabel;
			leftAttachment: XmATTACHFORM;
			rightAttachment: XmATTACHWIDGET;
			rightWidget: buttonOK;
			rightOffset: 10 ].

	description2Label setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: description1Label;
			topOffset: 0;
			leftAttachment: XmATTACHFORM ].

	materialsLabel setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: description2Label;
			leftAttachment: XmATTACHFORM ].

	materialsList parent setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: materialsLabel;
			leftAttachment: XmATTACHFORM;
			rightAttachment: XmATTACHWIDGET;
			rightWidget: buttonSelectAll;
			rightOffset: 15 ].

	buttonSelectAll setValuesBlock: [ :w |
		w
			topAttachment: XmATTACHWIDGET;
			topWidget: buttonCancel;
			topOffset: 38;
	"		leftAttachment: XmATTACHWIDGET;
			leftWidget: materialsList;
			leftOffset: 10;
	"		rightAttachment: XmATTACHFORM ].
!

ok: w clientData: clientData callData: callData
	"The OK button was pressed. Return the
	list of materials, who should be updated."

	self result: materialsList selectedItems.
	super ok: w clientData: clientData callData: callData. 
!

title
	"Answer the window title to use for the receiver."

	^'Update Materials'
! !

